@@ -20,7 +20,6 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#include <inttypes.h>
 #include <unistd.h>
 #include <getopt.h>
 #include <fcntl.h>
@@ -30,14 +29,6 @@
 #include <locale.h>
 #include <libintl.h>
 #include <sys/types.h>
-#include <sys/stat.h>
-
-#if MAJOR_IN_MKDEV
-#include <sys/mkdev.h>
-#elif MAJOR_IN_SYSMACROS
-#include <sys/sysmacros.h>
-/* else it's in sys/types.h, included above */
-#endif
 
 /* errors in <gtk.h> */
 #pragma GCC diagnostic push
@@ -51,17 +42,15 @@
 #include "ignore-value.h"
 #include "p2v.h"
 
-char **all_disks;
-char **all_removable;
 char **all_interfaces;
 int is_iso_environment = 0;
 int feature_colours_option = 0;
 int force_colour = 0;
-static const char *test_disk = NULL;
 
 static void udevadm_settle (void);
-static void set_config_defaults (struct config *config);
-static void find_all_disks (void);
+static void set_config_defaults (struct config *config,
+                                 const char * const *disks,
+                                 const char * const *removable);
 static void find_all_interfaces (void);
 
 enum { HELP_OPTION = CHAR_MAX + 1 };
@@ -121,12 +110,13 @@ display_short_options (const char *format)
 }
 
 static void
-display_long_options (const struct option *long_options)
+display_long_options (const struct option *long_options_p)
 {
-  while (long_options->name) {
-    if (STRNEQ (long_options->name, "long-options") && STRNEQ (long_options->name, "short-options"))
-      printf ("--%s\n", long_options->name);
-    long_options++;
+  while (long_options_p->name) {
+    if (STRNEQ (long_options_p->name, "long-options") &&
+        STRNEQ (long_options_p->name, "short-options"))
+      printf ("--%s\n", long_options_p->name);
+    long_options_p++;
   }
   exit (EXIT_SUCCESS);
 }
@@ -140,6 +130,8 @@ main (int argc, char *argv[])
   char **cmdline = NULL;
   int cmdline_source = 0;
   struct config *config = new_config ();
+  const char *test_disk = NULL;
+  char **disks, **removable;
 
   setlocale (LC_ALL, "");
   bindtextdomain (PACKAGE, LOCALEBASEDIR);
@@ -223,7 +215,24 @@ main (int argc, char *argv[])
 
   test_nbd_server ();
 
-  set_config_defaults (config);
+  /* Find all block devices in the system. */
+  if (test_disk) {
+    /* For testing and debugging purposes, you can use
+     * --test-disk=/path/to/disk.img
+     */
+    disks = malloc (2 * sizeof (char *));
+    if (disks == NULL)
+      error (EXIT_FAILURE, errno, "malloc");
+    disks[0] = strdup (test_disk);
+    if (disks[0] == NULL)
+      error (EXIT_FAILURE, errno, "strdup");
+    disks[1] = NULL;
+
+    removable = NULL;
+  } else
+    find_all_disks (&disks, &removable);
+
+  set_config_defaults (config, (const char **)disks, (const char **)removable);
 
   /* Parse /proc/cmdline (if it exists) or use the --cmdline parameter
    * to initialize the configuration.  This allows defaults to be pass
@@ -249,10 +258,12 @@ main (int argc, char *argv[])
       error (EXIT_FAILURE, 0,
              _("gtk_init_check returned false, indicating that\n"
                "a GUI is not possible on this host.  Check X11, $DISPLAY etc."));
-    gui_conversion (config);
+    gui_conversion (config, (const char **)disks, (const char **)removable);
   }
 
   guestfs_int_free_string_list (cmdline);
+  guestfs_int_free_string_list (removable);
+  guestfs_int_free_string_list (disks);
   free_config (config);
 
   exit (EXIT_SUCCESS);
@@ -265,7 +276,9 @@ udevadm_settle (void)
 }
 
 static void
-set_config_defaults (struct config *config)
+set_config_defaults (struct config *config,
+                     const char * const *disks,
+                     const char * const *removable)
 {
   long i;
   char hostname[257];
@@ -291,16 +304,18 @@ set_config_defaults (struct config *config)
   }
   config->guestname = strdup (hostname);
 
+  config->vcpu.phys_topo = false;
+
   /* Defaults for #vcpus and memory are taken from the physical machine. */
   i = sysconf (_SC_NPROCESSORS_ONLN);
   if (i == -1) {
     perror ("sysconf: _SC_NPROCESSORS_ONLN");
-    config->vcpus = 1;
+    config->vcpu.cores = 1;
   }
   else if (i == 0)
-    config->vcpus = 1;
+    config->vcpu.cores = 1;
   else
-    config->vcpus = i;
+    config->vcpu.cores = i;
 
   i = sysconf (_SC_PHYS_PAGES);
   if (i == -1) {
@@ -335,27 +350,10 @@ set_config_defaults (struct config *config)
   get_cpu_config (&config->cpu);
   get_rtc_config (&config->rtc);
 
-  /* Find all block devices in the system. */
-  if (!test_disk)
-    find_all_disks ();
-  else {
-    /* For testing and debugging purposes, you can use
-     * --test-disk=/path/to/disk.img
-     */
-    all_disks = malloc (2 * sizeof (char *));
-    if (all_disks == NULL)
-      error (EXIT_FAILURE, errno, "realloc");
-    all_disks[0] = strdup (test_disk);
-    if (all_disks[0] == NULL)
-      error (EXIT_FAILURE, errno, "strdup");
-    all_disks[1] = NULL;
-  }
-  if (all_disks)
-    config->disks = guestfs_int_copy_string_list (all_disks);
-
-  /* Find all removable devices in the system. */
-  if (all_removable)
-    config->removable = guestfs_int_copy_string_list (all_removable);
+  if (disks)
+    config->disks = guestfs_int_copy_string_list ((char **)disks);
+  if (removable)
+    config->removable = guestfs_int_copy_string_list ((char **)removable);
 
   /* Find all network interfaces in the system. */
   find_all_interfaces ();
@@ -369,159 +367,6 @@ set_config_defaults (struct config *config)
   config->output.storage = strdup ("/var/tmp");
 }
 
-static int
-compare (const void *vp1, const void *vp2)
-{
-  char * const *p1 = (char * const *) vp1;
-  char * const *p2 = (char * const *) vp2;
-  return strcmp (*p1, *p2);
-}
-
-/**
- * Get parent device of a partition.
- *
- * Returns C<0> if no parent device could be found.
- */
-static dev_t
-partition_parent (dev_t part_dev)
-{
-  CLEANUP_FCLOSE FILE *fp = NULL;
-  CLEANUP_FREE char *path = NULL, *content = NULL;
-  size_t len = 0;
-  unsigned parent_major, parent_minor;
-
-  if (asprintf (&path, "/sys/dev/block/%ju:%ju/../dev",
-                (uintmax_t) major (part_dev),
-                (uintmax_t) minor (part_dev)) == -1)
-    error (EXIT_FAILURE, errno, "asprintf");
-
-  fp = fopen (path, "r");
-  if (fp == NULL)
-    return 0;
-
-  if (getline (&content, &len, fp) == -1)
-    error (EXIT_FAILURE, errno, "getline");
-
-  if (sscanf (content, "%u:%u", &parent_major, &parent_minor) != 2)
-    return 0;
-
-  return makedev (parent_major, parent_minor);
-}
-
-/**
- * Return true if the named device (eg. C<dev == "sda">) contains the
- * root filesystem.  C<root_device> is the major:minor of the root
- * filesystem (eg. C<8:1> if the root filesystem was F</dev/sda1>).
- *
- * This doesn't work for LVs and so on.  However we only really care
- * if this test works on the P2V ISO where the root device is a
- * regular partition.
- */
-static int
-device_contains (const char *dev, dev_t root_device)
-{
-  struct stat statbuf;
-  CLEANUP_FREE char *dev_name = NULL;
-  dev_t root_device_parent;
-
-  if (asprintf (&dev_name, "/dev/%s", dev) == -1)
-    error (EXIT_FAILURE, errno, "asprintf");
-
-  if (stat (dev_name, &statbuf) == -1)
-    return 0;
-
-  /* See if dev is the root_device. */
-  if (statbuf.st_rdev == root_device)
-    return 1;
-
-  /* See if dev is the parent device of the root_device. */
-  root_device_parent = partition_parent (root_device);
-  if (root_device_parent == 0)
-    return 0;
-  if (statbuf.st_rdev == root_device_parent)
-    return 1;
-
-  return 0;
-}
-
-/**
- * Enumerate all disks in F</sys/block> and add them to the global
- * C<all_disks> and C<all_removable> arrays.
- */
-static void
-find_all_disks (void)
-{
-  DIR *dir;
-  struct dirent *d;
-  size_t nr_disks = 0, nr_removable = 0;
-  dev_t root_device = 0;
-  struct stat statbuf;
-
-  if (stat ("/", &statbuf) == 0)
-    root_device = statbuf.st_dev;
-
-  /* The default list of disks is everything in /sys/block which
-   * matches the common patterns for disk names.
-   */
-  dir = opendir ("/sys/block");
-  if (!dir)
-    error (EXIT_FAILURE, errno, "opendir");
-
-  for (;;) {
-    errno = 0;
-    d = readdir (dir);
-    if (!d) break;
-
-    if (STRPREFIX (d->d_name, "cciss!") ||
-        STRPREFIX (d->d_name, "hd") ||
-        STRPREFIX (d->d_name, "nvme") ||
-        STRPREFIX (d->d_name, "sd") ||
-        STRPREFIX (d->d_name, "ubd") ||
-        STRPREFIX (d->d_name, "vd")) {
-      char *p;
-
-      /* Skip the device containing the root filesystem. */
-      if (device_contains (d->d_name, root_device))
-        continue;
-
-      nr_disks++;
-      all_disks = realloc (all_disks, sizeof (char *) * (nr_disks + 1));
-      if (!all_disks)
-        error (EXIT_FAILURE, errno, "realloc");
-
-      all_disks[nr_disks-1] = strdup (d->d_name);
-
-      /* cciss device /dev/cciss/c0d0 will be /sys/block/cciss!c0d0 */
-      p = strchr (all_disks[nr_disks-1], '!');
-      if (p) *p = '/';
-
-      all_disks[nr_disks] = NULL;
-    }
-    else if (STRPREFIX (d->d_name, "sr")) {
-      nr_removable++;
-      all_removable = realloc (all_removable,
-                               sizeof (char *) * (nr_removable + 1));
-      if (!all_removable)
-        error (EXIT_FAILURE, errno, "realloc");
-      all_removable[nr_removable-1] = strdup (d->d_name);
-      all_removable[nr_removable] = NULL;
-    }
-  }
-
-  /* Check readdir didn't fail */
-  if (errno != 0)
-    error (EXIT_FAILURE, errno, "readdir: %s", "/sys/block");
-
-  /* Close the directory handle */
-  if (closedir (dir) == -1)
-    error (EXIT_FAILURE, errno, "closedir: %s", "/sys/block");
-
-  if (all_disks)
-    qsort (all_disks, nr_disks, sizeof (char *), compare);
-  if (all_removable)
-    qsort (all_removable, nr_removable, sizeof (char *), compare);
-}
-
 /**
  * Enumerate all network interfaces in F</sys/class/net> and add them
  * to the global C<all_interfaces> array.
@@ -573,5 +418,5 @@ find_all_interfaces (void)
     error (EXIT_FAILURE, errno, "closedir: %s", "/sys/class/net");
 
   if (all_interfaces)
-    qsort (all_interfaces, nr_interfaces, sizeof (char *), compare);
+    qsort (all_interfaces, nr_interfaces, sizeof (char *), compare_strings);
 }
